#!/bin/bash
# (c) Copyright 2007-2014 Hewlett Packard Development Company, L.P.
# Version A.12.00.00

#########################################################################
#
# This script contains PostgreSQL's defined functions that are used in the
# package toolkit control script which is used by Package Control Script.
#
# To be a part of a package,  this script has to be placed in the package
# directory.  The toolkit  control  script will check for this  script in
# the package  directory.  If it exists, the toolkit  control script will
# call it.
#
# Exit value  definition:  1 - a failure  occurs in one of the  function.
# Return control to the toolkit control script.
#
# REVISION HISTORY:
#   Version      mm/dd/yyyy   Changes
#   A.03.00      07/26/2006   1) Added code to remove stale pid file during
#                             package halt.
#
#   A.04.00      03/27/2008   1) Modular package support
# 
#   A.04.02.01   04/30/2012   1) Added variables PID_FILE, SERVER_PATH.  
#
#   A.04.02.02   10/19/2012   1) Changed the 'kill -3' command to 'pg_ctl
#                                stop' in the database stop sequence. 
#   A.12.00.00   09/15/2013   1) Removed maintanance mode support
#                             2) Removed legacy style package support
#                             3) Added idempotent operation support 
#########################################################################



######################### VARIABLES & PARAMETERS ########################
#
# Variables      : Initialize the variables with suitable values. 
#                  
# PKG_DIR= <Path of Package & toolkit configuration & control files>
#
#
# STARTUP_RETRIES = < Number > :
#
#                 This  variable  specifies  number  of  retries  in case
#                 pid-file is not found while  monitoring  or  startup.
#                 Default is set to "10" retries.
#
#
# exit_code = < Initial Non-Error State >
#
#                 This  variable  contain the exit status of this script.
#                 It  is  initially  assigned  "0",  indication  no-error
#                 condition.
#
# PID_FILE = < Name of the file with full path > 
# 
#                This variable holds path of the PID file of PostgreSQL
#                server.  
#
# SERVER_PATH = < Full path of the PostgreSQL server command >
#                PosstgreSQL server command path.
#  
# Arguments       :
#
#
# $1 = Action     :
#                 First parameter,  specified the operation to be carried
#                 out viz.  "start",  "stop"  or  "monitor".  Appropriate
#                 function is called for each operation.
#
#
#  Usage :  ./hapgsql.sh start | stop | monitor. 
#
#########################################################################


PATH=/bin:/sbin:/usr/bin:/usr/sbin:/bin:

PKG_DIR=${0%/*}


STARTUP_RETRIES=10 



exit_code=0


######################### FUNCTION HEADER ###############################
# Function Name  : check_return
# In Parameters  :
#
#             $1 : Exit Status of previously run command.
#
#             $2 : Message Number to be printed.
# Possible $2s   :
#                  0 - No error
#                  1 - Error in start_pgsql_server function
#                  2 - Error in stop_pgsql_server function
#                  3 - Error in monitor process
#                  4 - Error in Toolkit Configuration File
#                  default - unknown error
#
# Out   paramater: <Nil>
#
# Description    : Check return value of functions and logs appropriate
#                  message before exiting.
#
#########################################################################

function check_return
{
    if (( $1 != 0 ))
    then
    case $2 in
        1)
             echo "$(date '+%b %e %T') - Node \"$(hostname)\": ERROR: Function start_pgsql_server: Failed to start HA PostgreSQL daemons."
             stop_pgsql_server
             exit 1
        ;;
        2)
             echo "$(date '+%b %e %T') - Node \"$(hostname)\": ERROR: Function stop_pgsql_server: Failed to stop PostgreSQL daemons"
             echo "$(date '+%b %e %T') - Node \"$(hostname)\": Make sure to kill all lingering processes of this PostgreSQL package"
             exit_code=0
        ;;
        3)   echo "$(date '+%b %e %T') - Node \"$(hostname)\": ERROR: Could not Start the Monitor Function"
             exit 1
        ;;
        4)   echo "$(date '+%b %e %T') - Node \"$(hostname)\": ERROR: Improper parameters in Toolkit Configuration file"
             exit 1
        ;;
        *)
             echo "$(date '+%b %e %T') - Node \"$(hostname)\": ERROR: Failed, unknown error."
             exit 1
        ;;
    esac
    fi
}

# Confirm the DATA_DIRECTORY variable contains non-null string. 
if [ -z $DATA_DIRECTORY ]; then
    echo "$(date '+%b %e %T') - Node \"$(hostname)\": ERROR: DATA_DIRECTORY is not defined in hapgsql.conf"
    check_return 1 4
fi

# Confirm the LOG_FILE variable contains non-null string. 
if [ -z $LOG_FILE ]; then
    echo "$(date '+%b %e %T') - Node \"$(hostname)\": ERROR: LOG_FILE is not defined in hapgsql.conf"
    check_return 1 4
fi

# Assigning default value 30 to MONITOR_INTERVAL.
if [ -z $MONITOR_INTERVAL ]; then
    MONITOR_INTERVAL=30
fi

# Variables that holds the PID File and Server command path. 
PID_FILE="${DATA_DIRECTORY}/postmaster.pid"
SERVER_PATH=/usr/bin/postmaster


 
######################### FUNCTION HEADER ###############################
# Function Name  : start_pgsql_server
# In Parameters  : <Nil>
#
# Out   paramater: <Nil>
#                  
#
# Description    : Starts the PostgreSQL server with suitable parameter.
#
#########################################################################

function start_pgsql_server
{
    echo "$(date '+%b %e %T') - Node \"$(hostname)\": Starting PostgreSQL daemons."
     # Check if PostgreSQL daemon is already running on same DATA_DIRECTORY
    if [ -e "$DATA_DIRECTORY/postmaster.pid" ]
    then
        # Get the pid of the main postgres instance process from the postmaster.pid file
        pidofpro=`/usr/bin/head -n 1 $DATA_DIRECTORY/postmaster.pid`
        alive=`ps -p $pidofpro | grep $pidofpro`
        if [ "x$alive" != "x" ]
        then
             echo "$(date '+%b %e %T') - Node \"$(hostname)\": PostgresSQL Server is already running on $DATA_DIRECTORY"
             return 0
        fi
    fi
    # PostgresSQL Server is not running already, so start it
    su -l postgres -s /bin/bash -c "/usr/bin/pg_ctl -D $DATA_DIRECTORY -p $SERVER_PATH -l $LOG_FILE start > /dev/null 2>&1" < /dev/null
    ret_value=$?
    check_return $ret_value 1

    # Check if the pid file is ready.
    counter=0    
    while :; do
        if [ -f $PID_FILE ]; then
            break
        else
            if [ $counter -ge $STARTUP_RETRIES ]; then
                echo "$(date '+%b %e %T') - Node \"$(hostname)\": ERROR: Function start_pgsql_server: pgsql failed to create PID file $PID_FILE"
                check_return 1 1
            else
                sleep 1
                let counter="$counter+1"
            fi
        fi
    done
}

######################### FUNCTION HEADER ###############################
# Function Name  : stop_pgsql_server
# In Parameters  : <Nil>
#
# Out   paramater: <Nil>
#                  
#
# Description    : Stops the PostgreSQL server.
#
#########################################################################

function stop_pgsql_server
{
    local pid_name=""
    echo "$(date '+%b %e %T') - Node \"$(hostname)\": Stopping PostgreSQL daemons"
   
    if [ -f $PID_FILE ]; then
        read pid < $PID_FILE
        if [ -n $pid ]; then         
                pid_name=`ps --no-headers -o "%c" -p $pid`
                if [ "$pid_name" = "postmaster" ] ; then
                        su -l postgres -s /bin/bash -c "/usr/bin/pg_ctl -D $DATA_DIRECTORY -p $SERVER_PATH -l $LOG_FILE stop > /dev/null 2>&1" < /dev/null
                        pid_name=`ps --no-headers -o "%c" -p $pid` 
                        if [ "$pid_name" = "postmaster" ] ; then
                           # kill -s SIGKILL $pid # does not seem to work on RH5
                            kill -9 $pid
                            return_value=$?
                            check_return $return_value 2
                        fi
                elif [ -z $pid_name ]; then
                        echo "$(date '+%b %e %T') - Node \"$(hostname)\": postmaster daemon is not running."
                        # remove the stale pid file
                        echo "$(date '+%b %e %T') - Node \"$(hostname)\": Removing stale pid file $PID_FILE"
                        rm -f $PID_FILE

		else
                        echo "$(date '+%b %e %T') - Node \"$(hostname)\": ERROR: $pid is not the pid of postmaster daemon."
                        check_return 1 2
	        fi
         else
                echo "$(date '+%b %e %T') - Node \"$(hostname)\": ERROR: Function stop_pgsql_server: The $PID_FILE is corrupted."
                # remove the stale pid file
                echo "$(date '+%b %e %T') - Node \"$(hostname)\": Removing stale pid file $PID_FILE"
                rm -f $PID_FILE
		check_return 1 2
         fi
    else
        echo "$(date '+%b %e %T') - Node \"$(hostname)\": ERROR: Function stop_pgsql_server: The $PID_FILE does not exist."
        check_return 1 2
    fi
}

######################### STARTING POINT ###############################
#
# Description    : Invoke the Toolkit by calling appropriate function 
#                  depending on the inpur parameter.
#
# ----------------------------------------------------------------------
#  PARAMETER     |    Function Name 
# ----------------------------------------------------------------------
#   start        | start_pgsql_server
#                |
#   stop         | stop_pgsql_server
#                | 
#   monitor      | Call hapgsql.mon Script 
# ----------------------------------------------------------------------
#
#########################################################################


 case $1 in
 start)
    start_pgsql_server
 ;;

 stop)
    stop_pgsql_server
 ;;

 monitor)
    . $PKG_DIR/hapgsql.mon $PID_FILE $MONITOR_INTERVAL $RETRY_TIMES
    ret_value=$?
    check_return ret_value 3
 ;;
 
 *)
 echo -e "$(date '+%b %e %T') - Node \"$(hostname)\": Invalid Paramter specified\n"
 echo -e "\t\t- Usage: ${0} [ start | stop | monitor ]"
 exit_code=1
 ;;
 esac

exit $exit_code
